@@ -143,7 +143,16 @@ sub init_allowed_keys {
 	    TYPE => 'STRING',
 	    VARNAME => 'mailprog',
 	    GROUP => 'Programs',
-	    CHECK => $validate_program,
+	    CHECK => sub {
+		my $conf = shift;
+		my $entry = shift;
+		my $key = $entry->{'NAME'};
+
+		# Only validate if needed.
+		if ($conf->get('MAILTO')) {
+		    $validate_program->($conf, $entry);
+		}
+	    },
 	    DEFAULT => '/usr/sbin/sendmail',
 	    HELP => 'Program to use to send mail'
 	},
@@ -169,6 +178,13 @@ sub init_allowed_keys {
 	    DEFAULT => $native_arch,
 	    HELP => 'Build architecture (Arch we are building on).'
 	},
+	'BUILD_PROFILES'        => {
+	    TYPE => 'STRING',
+	    VARNAME => 'build_profiles',
+	    GROUP => 'Build options',
+	    DEFAULT => $ENV{'DEB_BUILD_PROFILES'} || '',
+	    HELP => 'Build profiles. Separated by spaces.'
+	},
 	'HOSTNAME'				=> {
 	    TYPE => 'STRING',
 	    GROUP => '__INTERNAL',
